package BATCH2UR::Generate;

use 5.008;
use strict;
use warnings;

use Template;
use BATCH2UR::PBSParser;
use BATCH2UR::LSFParser;
#use BATCH2UR::GlobusParser; # for pre-GT5
use BATCH2UR::Globus5Parser; # for GT5
use BATCH2UR::LcasParser;

use BATCH2UR::LcmapsParser;

use BATCH2UR::GridmapParser;
use BATCH2UR::TorqueParser;
use BATCH2UR::SGEParser;
use BATCH2UR::CondorParser;
use BATCH2UR::MakeGridmapParser;
use BATCH2UR::Config;
use BATCH2UR::ConversionFunctions;
use File::Basename;
use Data::Dumper;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(batchToUR batchToDumper);
our $VERSION = '0.7';

#-----------------------------------------------------------------------
# Generation functions
#-----------------------------------------------------------------------
sub batchToUR {
    my $file=shift;
    my $ur;
    my $urDir=$config->UrFileDir;
    my $urTemplateDir=$config->UrTemplateDir;
    my $templateFile;

    unless ($file) {
	print "No file specified for conversion.\n";
	return 0;
    };
    unless ($urDir) {
	print "No UrDir specified in batch2ur.conf.\n";
	return 0;
    };
    # Choose which parser to use and which template.....
    for (lc($config->BatchSystem)) {
      if ( m/lsf/o ) {
        $ur = BATCH2UR::LSFParser::parse($file);
        $templateFile="$urTemplateDir/templates/ur-lsf.xml";
      }elsif ( m/pbs/o ){
	die "PBS - $config->BatchSystem";
        $ur = BATCH2UR::PBSParser::parse($file);
        $templateFile="$urTemplateDir/templates/ur-pbs.xml";
      }elsif ( m/torque/o ) {
        $ur = BATCH2UR::TorqueParser::parse($file);
        $templateFile="$urTemplateDir/templates/ur-torque.xml";
      }elsif ( m/sge/o ) {
        $ur = BATCH2UR::SGEParser::parse($file);
        $templateFile="$urTemplateDir/templates/ur-sge.xml";
      }elsif ( m/condor/o ) {
        $ur = BATCH2UR::CondorParser::parse($file);
        $templateFile="$urTemplateDir/templates/ur-condor.xml";
      }else {
    	die "Unknown batch system: $config->BatchSystem";
      };
    };

    for my $jobid (keys %{$ur} ) {
	# Lookup info from datasources
	my %globus;
        my %lcas;
        my %gridmap;
        my %makegridmap;
	
	my %lcmaps;

        #print ">> Processing $jobid - ".$ur->getRecord($jobid)->LocalUserId(). "\n";

        # Attempt to lookup as Globus job
        %globus=Globus5Parser($jobid);

        # Lcas logs
        if ( $globus{jobTypeId} ) {
          %lcas=LcasParser($globus{jobTypeId});
        };

        #Non globus jobs need to find DN and voinfo via other means
	if ( ! %globus ) {
          %gridmap=GridmapParser($ur->getRecord($jobid)->LocalUserId());
          %makegridmap=MakeGridmapParser($ur->getRecord($jobid)->LocalUserId()); #voinfo
        }

        print "\n";
		#LCMAPS VO Data
        #print "Globus...\n";
		#while ((my $key, my $value) = each(%globus)){
     	#	print $key.", ".$value."\n";
		#}

        #print " DN: ".$globus{dn}."\n";
		#print " Job submitted: ".$globus{timeGlobusSubmitted}."\n";

	%lcmaps=LcmapsParser($jobid);

        # Merge all sources
	my %input=(
		   JobId=>$jobid,
		   createTime=>epochToDate(time),
		   %{$ur->{$jobid}},
		   %globus,
                   %lcas,
                   %gridmap,
                   %makegridmap,
		   %lcmaps,
		   );

	#Ensure RUS required fields are present
	if ( ! $input{"dn"} ) {
	    $input{"dn"}="unknown";
	};
	if ( ! $input{"MachineName"} ) {
	    $input{"MachineName"}=$config->MachineName;
	};
	if ( ! $input{"SubmitHost"} ) {
	    $input{"SubmitHost"}=$config->SubmitHost;
	};

	#Generate files
	my $output;
	my $template=Template->new({ABSOLUTE=>1});
	$template->process($templateFile, \%input, \$output)
	    || die $template->error();
	open UR, ">$urDir/upload/$jobid.xml" or
	    die "Cannot open $urDir/upload/$jobid.xml. $!";
	chmod 0600,"$urDir/upload/$jobid.xml" or
	    die "Cannot change permissions on $urDir/upload/$jobid.xml. $!";
	print UR $output;
	close UR;
    }
    return 1;
};

sub batchToDumper {
    my $file=shift;
    my $ur;
    unless ($file) {
	print "No file specified for conversion.\n";
	return 0;
    };
    # Choose which parser to use.....
    if ( $config->BatchSystem eq "LSF" ) {
    	$ur = BATCH2UR::LSFParser::parse($file);
    }elsif ( $config->BatchSystem eq "PBS" ) {
    	$ur = BATCH2UR::PBSParser::parse($file);
    }elsif ( $config->BatchSystem eq "SGE" ) {
    	$ur = BATCH2UR::SGEParser::parse($file);
    }elsif ( $config->BatchSystem eq "Torque"  ) {
        $ur = BATCH2UR::TorqueParser::parse($file);
    }elsif ( $config->BatchSystem eq "Condor"  ) {
        $ur = BATCH2UR::CondorParser::parse($file);
    }else {
    	die "Unknown batch system: $config->BatchSystem";
    };
    for my $jobid (keys %{$ur} ) {
	#Try to lookup in Globus logs
	my %globus=Globus5Parser($jobid);
	if ( ! %globus ) {
	    #Resort to Gridmapfile if info not found in Globus logs
	    %globus=GridmapParser($ur->getRecord($jobid)->LocalUserId());
	};
	my %input=(
		   JobId=>$jobid,
		   createTime=>epochToDate(time),
		   %{$ur->{$jobid}},
		   %globus,
		   );
	#Ensure RUS required fields are present
	if ( ! $input{"dn"} ) {
	    $input{"dn"}="unknown";
	};
	if ( ! $input{"MachineName"} ) {
	    $input{"MachineName"}=$config->MachineName;
	};
	if ( ! $input{"SubmitHost"} ) {
	    $input{"SubmitHost"}=$config->SubmitHost;
	};
	#Upload to Database
	#Dump data to work out DB schema
	$Data::Dumper::Indent = 3;
	$Data::Dumper::Pair = " : ";
	print Dumper(\%input), "\n";
    }
    return 1;
};


1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

BATCH2UR::Generate - Perl module to generate usage record's from Batch system and Globus.

=head2 EXPORT

batchToUR - generate usage record from batch job info
batchToDumper - for debugging will print details of batch job

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
