package BATCH2UR::Globus5Parser;

use 5.008;
use strict;
use warnings;
use BerkeleyDB;
use MLDBM qw(BerkeleyDB::Hash Storable);
use BATCH2UR::Config;
use BATCH2UR::ConversionFunctions;

# for GT5
use Globus::Core::Paths;
use DBI qw(:sql_types);

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(Globus5Parser);

my $db_cache=$config->BatchDatabaseCache; # Default is 64MB
my $idx_cache=$config->BatchIndexCache; # Default is 32MB
my $databaseDir=$config->BatchDatabaseDir;
my $batchSystem=lc($config->BatchSystem); # Always use lowercase

if ( $batchSystem eq "torque" ) {
  $batchSystem = "pbs"; # globus log accounting makes no distinction between pbs and torque
};

my $audit_directory=get_audit_directory();
if ($audit_directory eq '') {
    die "No -audit-directory specified in job-manager.conf. " .
        "Nothing to do\n";
}

# Parse configuration
my $conf_path = "$Globus::Core::Paths::sysconfdir/globus/gram-audit.conf";
(my %conf = parse_conf($conf_path)) || exit(1);

# Deal with DB driver
my $dbh = dbconnect(\%conf);
if (!defined($dbh)) {
    die "Could not connect to GRAM audit database\n";
}

# for GT5
sub Globus5Parser {
    my $jobid = shift;

    my $sth = $dbh->prepare("SELECT "
        . "job_grid_id,"
        . "local_job_id,"
        . "subject_name,"
        . "username,"
        . "idempotence_id,"
        . "creation_time,"
        . "queued_time,"
        . "stage_in_grid_id,"
        . "stage_out_grid_id,"
        . "clean_up_grid_id,"
        . "globus_toolkit_version,"
        . "resource_manager_type,"
        . "job_description,"
        . "success_flag,"
        . "finished_flag"
        . " FROM gram_audit_table"
        . " WHERE gram_audit_table.local_job_id='" . $jobid . "'"
    );

    my $rc = $sth->execute();
#    while (my @row = $sth->fetchrow_array) {
#        print DBI::neat_list(\@row, 2560, ",") . "\n";
#    }

    my $result = $sth->fetchrow_hashref;
    my %globus = ();
    if (defined $result) {
        %globus = (
            "timeGlobusSubmitted" => globusAuditToDate($result->{queued_time}),
            "dn" => $result->{subject_name},
            "jobTypeId" => $result->{resource_manager_type}
        );
    }

    return %globus;
};

# get_audit_directory
#
# Parse the default job manager configuration file. Return the argument
# to -audit-directory if present in the file, or an empty string.
#
# \return
#     Path of the audit directory, or an empty string if not present in the
#     configuration
sub get_audit_directory {
    my $jmconf = "$Globus::Core::Paths::sysconfdir/globus/globus-gram-job-manager.conf";
    my $audit_directory = '';
    local(*F);
    open(F, "<$jmconf");
    while(<F>) {
        if (m/-audit-directory\s+(\S+)\s+/) {
            $audit_directory = Globus::Core::Paths::eval_path($1);
            last;
        }
    }
    close(F);
    return $audit_directory;
};

# Connect to database described in configuration hash %conf.
#
# \arg %conf
#     Hash containing DRIVER, DATABASE, USERNAME, and PASSWORD keys.
# \return
#     DBI handle connected to the selected database or undef.
sub dbconnect
{
    my $conf = $_[0];
    my $dbh;
    my $sth;
    my @available_drivers = DBI->available_drivers();
    my $driver_ok = 0;

    for (my $i = 0; $i < scalar(@available_drivers); $i++)
    {
        if ($available_drivers[$i] eq $conf->{'DRIVER'})
        {
            $driver_ok = 1;
        }
    }
    if (!$driver_ok)
    {
        print STDERR "Error connecting to database: driver "
                   . "\"$conf->{DRIVER}\" not installed\n";
        return undef;
    }

    $dbh = DBI->connect(
            "dbi:$conf->{DRIVER}:$conf->{DATABASE}",
            $conf->{USERNAME},
            $conf->{PASSWORD});

    if (!defined($dbh))
    {
        print STDERR "Error connecting to database\n";
        return undef;
    }

    return $dbh;
};

# parse_conf
#
# Parse configuration file named by $path. The configuration file
# is expected to contain key-value pairs separated by :
#
# \arg $path
#     Path to GRAM audit configuration file
# \return
#     Hash containing ($key, $value) pairs from $path
sub parse_conf
{
    my $path = shift;
    my %result = ();

    local(*CONF);

    if (!open(CONF, "<$path"))
    {
        print STDERR "Error opening configuration file \"$path\"\n";
        return ();
    }

    while (<CONF>)
    {
        my ($key, $value);
        chomp;
        ($key, $value) = split(/:/, $_, 2);

        $result{$key} = Globus::Core::Paths::eval_path($value);
    }
    return %result;
}

END {
    if ($dbh)
    {
        $dbh->disconnect;
    }
}

1;

__END__
=head1 NAME

BATCH2UR::Globus5Parser - Reads GRAM5 database of captured Globus audit logs.

=head2 EXPORT

Globus5Parser(jobid) - get hash of Globus data by jobid.

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>
Steve Crouch, E<lt>s.crouch@software.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
