package BATCH2UR::UsageRecord;

use strict;
use warnings;
use base qw(Class::Accessor);

#----------------------------------------------------------------------
# Error Codes
#----------------------------------------------------------------------
use constant NEED_ARRAY_REF => -10;
use constant NEED_HASH_REF  => -20;

#----------------------------------------------------------------------
# Constructor
#----------------------------------------------------------------------
sub new {
    my $self = {};    
    bless ($self);
    return ($self);
}


#----------------------------------------------------------------------
# Automatically create simple Get/Set Methods for this class members
#----------------------------------------------------------------------
__PACKAGE__->mk_accessors('JobId',
			  'LocalUserId',
			  'JobName',
			  'Status',
			  'wallTimeRequested',
			  'cpuTimeRequested',
			  'timePbsSubmitted',
			  'timePbsQueued',
			  'timePbsExecutable',
              'timeTorqueSubmitted',
			  'timeTorqueQueued',
			  'timeTorqueExecutable',
              'timeLsfSubmitted',
			  'timeLsfQueued',
			  'timeLsfExecutable',
              'timeSgeSubmitted',
              'timeCondorSubmitted',
			  'WallDuration',
			  'CpuDuration',
			  'Processors',
			  'EndTime',
			  'StartTime',
			  'Queue',
			  'NodeCount',
			  'logDate',
			  );


## Comments by JW
# These are not default elements in the UR schema
# So perhaps should not be in this class....
#			  'timeSubmitted',
#			  'timeQueued',
#			  'timeExecutable',
#  Also shoule these fields be batch system dependent
#  e.g., timePbsSubmitted.

## Comments by MF
# Changed back to time batch system dependent names
# Actual reporting behaviour can be configured at the template level if
# necessary.
#
# lower case letters indicate an attribute (or type) in the UR spec
# upper case letters indicate full blown element content.
#

#----------------------------------------------------------------------
# Other Get/Set Methods (Host, MemoryUsed, VirtMemUsed)
#----------------------------------------------------------------------
sub Host {
    my $self = shift;
    my @host = @_;
    #
    if (@host) {
	$self->{'Host'} = \@host;
    }
    else {
	return $self->{'Host'}; 
    }    
}


sub memoryUsed {
    my $self = shift;
    my $mem  = shift;
    my $unit = shift;
    # We don't used define here as a value of zero is rejected by the RUS server
    if ($mem && $unit) {
	$self->{'memoryUsed'} = { 'quantity' => $mem, 
				  'unit'     => $unit };
    }
    else {
	return $self->{'memoryUsed'};
    }
}


sub virtualMemoryUsed {
    my $self = shift;
    my $mem  = shift;
    my $unit = shift;
    # We don't used define here as a value of zero is rejected by the RUS server
    if ($mem && $unit) {
	$self->{'virtualMemoryUsed'} = { 'quantity' => $mem, 
					 'unit'     => $unit };
    }
    else {
	return $self->{'virtualMemoryUsed'};
    }
    
}

#----------------------------------------------------------------------
# Print UR Object (for debugging, sanity checking etc)
#----------------------------------------------------------------------
sub toString {
    my $self = shift;
    #
    foreach my $key (sort(keys( %{$self} ))) {
	#
	print "$key\t";
	print "\t"      if (length($key) < 8);
	print "\t"      if (length($key) < 16);
	print "=> ";
	#
	if (ref($self->{$key}) eq 'ARRAY') {
	    print @{ $self->{$key} };
	}
	elsif (ref($self->{$key}) =~ /HASH/) {
	    print "[ mem = ", 
	    $self->{$key}->{quantity},
	    ", unit = ",  
	    $self->{$key}->{unit},
	    " ]";
	}
	else {
	    print $self->{$key};
	}
	# 
	print "\n";
    }
}


1;

__END__

=head1 NAME

BATCH2UR::UsageRecordList - Class for storing Job Scheduler Usage Records

=head1 SYNOPSIS

  use BATCH2UR::UsageRecord;

  # Add Data to UsageRecord
  $ur->localUserID('ngs005');
  $ur->status('complete');
  $ur->nodeCount(28);
  $ur->toString();


=head1 ABSTRACT

BATCH2UR::UsageRecord is a class for storing individual Job Usage Records.


=head1 DESCRIPTION


=head1 METHODS

=head2 new

Creates a new UsageRecord object.

=head2 jobID(string)

Sets the Job ID, or returns the JobID if called with no arguement.

=head2 localUserID(string)

Sets the local user ID, or returns the user ID if called with no arguement.

=head2 jobName(string)

Sets the Job name, or returns the Job name if called with no arguement.

=head2 status(string)

Sets the Job status, or returns the Job status if called with no arguement.

=head2 wallTimeRequested(time)

Sets the amount of walltime requested, or returns the amount of walltime requested
 if called with no arguement.

=head2 cpuTimeRequested(time)

Sets the Job CPU Time requested, or returns the length of CPU Time requested 
if called with no arguement.

=head2 timeSubmitted(time)

Sets the Job submission time, or returns the Job submission time if called with 
no arguement.

=head2 timeQueued(time)

Sets the Job Queue time, or returns the Job queue time if called with no arguement.

=head2 timeExecutable(time)

Sets the Job status, or returns the Job status if called with no arguement.

=head2 wallDuration(time)

Sets the Job Wall time, or returns the Job status if called with no arguement.

=head2 cpuDuration(time)

Sets the amount of CPU time used, or returns the CPU time if called with no 
arguement.

=head2 processors(int)

Sets the number of processors, or returns the number of processors if called with
no arguement.

=head2 endTime(string)

Sets the End time, or returns the end time if called with no arguement.

=head2 startTime(string)

Sets the start time, or returns the start time if called with no arguement.

=head2 queue(string)

Sets the Job queue name, or returns the Job queue name if called with no arguement.

=head2 nodeCount(int)

Sets the number of nodes used, or returns the number of nodes if called with no 
arguement.

=head2 host(@array)

Sets an array containing a list of the execution node names ?, or returns
this list if called with no arguments.

=head2 memoryUsed(amount, unit)

Sets the Amount of memory used and the units for this quantity, or returns a 
hash reference containing the amount of memory used and the corresponding 
units if called with no arguments.

=head2 virtualMemoryUsed(amount, unit)

Sets the Amount of memory used and the units for this quantity, or returns a 
hash reference containing the amount of memory used and the corresponding 
units if called with no arguments.

=head2 EXPORT


=head1 SEE ALSO

BATCH2UR::UsageRecordList - a module to create conatiners for storing UsageRecord 
objects.

=head1 AUTHOR

Jon Wakelin, E<lt>isxjw@bris.ac.ukkE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Jon Wakelin

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
